#------------------------------------------------
# PURPOSE: Check MCMC simulations and residuals.
#------------------------------------------------
# define target result folders
#targets=paste('Meyras/',
#              list.files('Meyras','^SD*'),'/',sep='') # all Meyras SD folders
#targets=paste('Meyras/',
#              list.files('Meyras','^SPD*'),'/',sep='') # all Meyras SPD folders
targets=paste('Wairau/',
              list.files('Wairau','^SPD*'),'/',sep='') # all Wairau SPD folders
n=length(targets)
# define names of MCMC and residual files
mcmcFile='Results_MCMC_Cooked.txt'
residualFile='Results_Residuals.txt'
# define names of result pdf files
mcmcResFile='Check_MCMC.pdf'
residualResFile='Check_Residuals.pdf'
# define number of columns used to plot MCMC samples
ncol=5

for(i in 1:n){
  #------------------------------------------------
  # Check MCMC simulations
  #------------------------------------------------
  # read MCMC file
  file=paste(targets[i],mcmcFile,sep='')
  mcmc=read.table(file,header=T)
  # find LogPost column and deduce number of parameters
  indx=which(colnames(mcmc)=="LogPost")
  npar=indx-1
  # create plot
  nrow=ceiling(npar/ncol) # number of rows needed for all parameters
  resFile=paste(targets[i],mcmcResFile,sep='')
  pdf(resFile,width=10,height=2*nrow,useDingbats=F)
  par(mfrow=c(nrow,ncol))
  for(j in 1:npar){
    # MCMC trace for each inferred parameter
    plot(mcmc[,j],type='l',xlab='iteration',ylab=names(mcmc)[j])
  }
  dev.off()  
  #------------------------------------------------
  # Check residuals
  #------------------------------------------------
  # read residuals file
  file=paste(targets[i],residualFile,sep='')
  resid=read.table(file,header=T)
  # define non-missing values mask
  mask=(resid$Y1_stdres!=-9999)
  if(sum(mask)>0){
    # create plot
    resFile=paste(targets[i],residualResFile,sep='')
    pdf(resFile,width=10,height=3,useDingbats=F)
    par(mfrow=c(1,3))
    # standardized residuals in order of appearance
    plot(resid$Y1_stdres[mask],
         pch=19,xlab='index',ylab='standardized residual')
    # simulated Q vs. standardized residuals in order of appearance
    plot(resid$Y1_sim[mask],resid$Y1_stdres[mask],
         pch=19,log='x',
         xlab='rating curve discharge',
         ylab='standardized residual')
    # N(0,1) qq-plot
    qqnorm(resid$Y1_stdres[mask],pch=19)
    qqline(resid$Y1_stdres[mask],col='red',
           distribution=function(p) qnorm(p,0,1))
    dev.off()  
  }
}