
#########################################################################################################
############################################## Function #################################################
#########################################################################################################
scatter.Shifts.ggplot=function(###### data needed
  matDistrib
  ,data.Maxpost
  ,file.path
  ###### layout and save
  ,wind.height=30
  ,wind.width=30
  ,layout=NULL
  ###### distribution settings
  ,prior.param=matrix(c(c(14.147,-0.6,1.5,26.52,-0.6,1.6667,31.82,1.2,1.6667)
                        ,c(5.01,0.5,0.025,8.4,0.5,0.025,10.93,0.2,0.025)),ncol=2)
  ,loi.type=c(rep(TRUE,9))
  ###### hex settings
  ,hex.size=2
  ,size.axis.text.y=20
  ,size.axis.title.y=20
  ,face.axis.text.y="bold"
  ###### correlation settings
  ,base_size=20
  ,coeff.cor.size=20
  ,hex.bins=10
  ,low.cor="#fff5f0",high.cor="#67000d"
  ###### densities settings
  ,alpha.dens=c(0.5,0.5)
  ,coeff.dens=rep(3,9)
  ,size.gene=2
  ,size.vline=matrix(2,ncol=2,nrow=9)
  ,colour.dens=c("#ef8a62","#67a9cf")
  ,linetype.vline=matrix("longdash",ncol=2,nrow=9)
  ,wind.lim=matrix(c(rep(-1,2),rep(1,2)),ncol=2)
  ###### annotate settings
  ,sizeDens=rep(20,9)
  ,coeff.annote=matrix(0.85,ncol=2,nrow=9)
  ,col.annotate=rep("#737373",9)
  ,label=c("a[1]",as.character(expression(b[1]^{(1)})),"c[1]"
           ,"a[2]",as.character(expression(b[2]^{(1)})),"c[2]"
           ,"a[3]",as.character(expression(b[3]^{(1)})),"c[3]")
  ###### theme settings
  ,axis.label=c(expression(paste("Coefficient ",a[1]," [",m^frac(3,2),".",s^-1,"]",sep="")
                           ,paste("Offset ",b[1]^{(1)}," [m]",sep="")
                           ,paste("Exponent ",c[1]," [-]",sep="")
                           ,paste("Coefficient ",a[2]," [",m^frac(4,3),".",s^-1,"]",sep="")
                           ,paste("Offset ",b[2]^{(1)}," [m]",sep="")
                           ,paste("Exponent ",c[2]," [-]",sep="")
                           ,paste("Coefficient ",a[3]," [",m^frac(4,3),".",s^-1,"]",sep="")
                           ,paste("Offset ",b[3]^{(1)}," [m]",sep="")
                           ,paste("Exponent ",c[3]," [-]",sep="")))
  ,size.axis.text.x=20,face.axis.text.x="bold",size.axis.title.x=20,title.do=FALSE){
  
  ###### library loading
  if(!require(ggplot2)){
    install.packages("ggplot2")
    require(ggplot2)
  }
  if(!require(pryr)){
    install.packages("pryr")
    require(pryr)
  }
  if(!require(grid)){
    install.packages("grid")
    require(grid)
  }
  if(!require(hexbin)){
    install.packages("hexbin")
    require(hexbin)
  }
  
  ###### data settings
  nsim=dim(matDistrib)[1]
  nvar=dim(matDistrib)[2]
  Quant.MCMC=apply(matDistrib,MARGIN=2,quantile,probs=c(0.025,0.975))
  
  ### layout
  # Make the panel
  # ncol: Number of columns of plots
  # nrow: Number of rows needed, calculated from # of cols
  if(is.null(layout)){
    layout <- matrix(seq(1, nvar*nvar),
                     ncol = nvar, nrow = nvar,byrow = TRUE)
  }
  
  ##### PDF opening
  pdf(file.path,wind.width,wind.height,useDingbats=F)
  
  # Set up the page
  grid.newpage()
  pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
  
  i=1
  j=1
  i=i+1
  j=j+1
  
  for(i in 1:nvar){
    for(j in 1:nvar){
      if(i==j){ #### Diagonal i. e. variable name
        matDist.vect=data.frame(x=matDistrib[,i])
        n_i=sum(loi.type)
        if(loi.type[i]){
          x.var=seq(min(matDist.vect$x,prior.param[i,1]-coeff.dens[i]*prior.param[i,2])
                    ,max(matDist.vect$x,prior.param[i,1]+coeff.dens[i]*prior.param[i,2]),length=nsim)
          y.var=dnorm(x.var,mean=prior.param[i,1],sd=prior.param[i,2])
          matprior=data.frame(x=x.var,y=y.var)
          min.x=min(density(matDist.vect$x)$x,x.var)
          max.x=max(density(matDist.vect$x)$x,x.var)
          min.y=min(density(matDist.vect$x)$y,y.var)
          max.y=max(density(matDist.vect$x)$y,y.var)
        }else{
          min.x=min(density(matDist.vect$x)$x)
          max.x=max(density(matDist.vect$x)$x)
          min.y=min(density(matDist.vect$x)$y)
          max.y=max(density(matDist.vect$x)$y)
        }
        density.post=density(matDist.vect$x)
        inter.post=which(density.post$x>=Quant.MCMC[1,i])[1]:(which(density.post$x>Quant.MCMC[2,i])[1]-1)
        matPost.dens=data.frame(x=density.post$x,y=density.post$y)
        m=ggplot(matDist.vect)+
          annotate("text",x=min.x+(max.x-min.x)*coeff.annote[i,1]
                   ,y=max.y*coeff.annote[i,2],label=label[i],colour=col.annotate[i]
                   ,size=sizeDens[i],parse=TRUE)+
          theme_bw()+
          theme(axis.text.y=element_blank()
                ,axis.ticks.y=element_blank(),legend.position="none",axis.title.y=element_blank()
                ,axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank()
                ,panel.grid.major=element_blank(),panel.grid.minor=element_blank())
      }else if(i>j){ #### Correlation value, upper part
        matCorr.tmp=data.frame(matDistrib[,c(i,j)])
        colnames(matCorr.tmp)=c("x","y")
        
        corr.size=coeff.cor.size*abs(cor(matCorr.tmp)[1,2])
        
        m=ggplot(data=matCorr.tmp)+
          geom_text(aes(x=(min(x)+max(x))/2,y=(min(y)+max(y))/2,label=round(cor(x,y),2),colour=abs(round(cor(x,y),2)))
                    ,size=corr.size)+
          scale_colour_gradient(low=low.cor,high=high.cor,limits=c(0,1))+
          theme_bw()+
          theme(axis.text.x=element_blank(),axis.text.y=element_blank()
                ,axis.ticks=element_blank(),legend.position="none",axis.title=element_blank()
                ,panel.grid.major=element_blank(),panel.grid.minor=element_blank())
      }else{
        matCorr.tmp=data.frame(matDistrib[,c(i,j)])
        colnames(matCorr.tmp)=c("x","y")
        m=ggplot(data=matCorr.tmp)+
          # hexbinplot(y~x, data=matCorr.tmp)+
          geom_hex(aes(x=x,y=y),size=hex.size,bins=hex.bins)+
          # geom_hex(aes(x=x,y=y,fill=..density..,colour=..density..),size=hex.size,bins=hex.bins)+
          scale_colour_gradient(low="#74c476",high="#00441b")+
          scale_fill_gradient(low="#74c476",high="#00441b")+
          coord_cartesian(xlim=wind.lim[1,],ylim=wind.lim[2,], expand = FALSE)+
          theme_bw(base_size=base_size)
        if(i==1 && j!=nvar){
          if(title.do){
            m=m+
              eval(substitute(ylab(x),list(x=axis.label[j])))+
              theme(axis.text.x=element_blank(),axis.text.y=element_text(size=size.axis.text.y)
                    ,axis.ticks.x=element_blank(),legend.position="none",axis.title.x=element_blank()
                    ,axis.title.y=element_text(size=size.axis.title.y,face=face.axis.text.y))
          }else{
            m=m+
              eval(substitute(ylab(x),list(x=axis.label[j])))+
              theme(axis.text.x=element_blank(),axis.text.y=element_blank()
                    ,axis.ticks=element_blank(),legend.position="none",axis.title=element_blank())
          }
        }else if(i==1 && j==nvar){
          if(title.do){
            m=m+
              eval(substitute(ylab(x),list(x=axis.label[j])))+
              eval(substitute(xlab(x),list(x=axis.label[i])))+
              theme(axis.text.y=element_text(size=size.axis.text.y)
                    ,axis.text.x=element_text(size=size.axis.text.y)
                    ,legend.position="none"
                    ,axis.title.x=element_text(size=size.axis.title.y,face=face.axis.text.y)
                    ,axis.title.y=element_text(size=size.axis.title.y,face=face.axis.text.y))
          }else{
            m=m+
              eval(substitute(ylab(x),list(x=axis.label[j])))+
              eval(substitute(xlab(x),list(x=axis.label[i])))+
              theme(axis.text.x=element_blank(),axis.text.y=element_blank()
                    ,axis.ticks=element_blank(),legend.position="none",axis.title=element_blank())
          }
        }else if(i!=1 && j==nvar){
          if(title.do){
            m=m+
              eval(substitute(xlab(x),list(x=axis.label[i])))+
              theme(axis.text.y=element_blank()
                    ,axis.text.x=element_text(size=size.axis.text.y)
                    ,axis.ticks.y=element_blank()
                    ,legend.position="none"
                    ,axis.title.x=element_text(size=size.axis.title.y,face=face.axis.text.y)
                    ,axis.title.y=element_blank())
          }else{
            m=m+
              eval(substitute(xlab(x),list(x=axis.label[i])))+
              theme(axis.text.x=element_blank(),axis.text.y=element_blank()
                    ,axis.ticks=element_blank(),legend.position="none",axis.title=element_blank())
          }
        }else{
          m=m+
            theme(axis.text.x=element_blank(),axis.text.y=element_blank()
                  ,axis.ticks=element_blank(),legend.position="none",axis.title=element_blank())
        }
      }
      matchidx <- as.data.frame(which(layout == i+(j-1)*nvar, arr.ind = TRUE))      
      print(m, vp = viewport(layout.pos.row = matchidx$row,
                             layout.pos.col = matchidx$col))
    }
  }
  dev.off()
}


#########################################################################################################
############################################ Data Loading ###############################################
#########################################################################################################

load("Priors/priorMeyras.RData")




#### Figure4-a

scatter.Shifts.ggplot(matDistrib = LocalGlobal
                      ,data.Maxpost = rep(0,8)
                      ,file.path = paste("ScatterPlot_LocalGlobal.pdf",sep="")
                      ###### distribution settings
                      ,prior.param=matrix(c(rep(0,8),rep(0.5,8)),ncol=2)
                      ,loi.type=c(rep(TRUE,8))
                      ,coeff.cor.size=40
                      ,low.cor="black",high.cor="black"
                      ###### densities settings
                      ,coeff.dens=rep(3,8)
                      ,size.gene=2
                      ,size.vline=matrix(2,ncol=2,nrow=8)
                      ,colour.dens=c("#ef8a62","#67a9cf")
                      ,linetype.vline=matrix("longdash",ncol=2,nrow=8)
                      ,wind.lim=matrix(c(rep(-6.1,2),rep(6.1,2)),ncol=2)
                      ###### annotate settings
                      ,sizeDens=rep(40,8)
                      ,coeff.annote=matrix(0.85,ncol=2,nrow=8)
                      ,col.annotate=rep("#737373",8)
                      ,label=c(as.character(expression(delta*l^{(2)})),as.character(expression(delta*g^{(2)}))
                               ,as.character(expression(delta*l^{(3)})),as.character(expression(delta*g^{(3)}))
                               ,as.character(expression(delta*l^{(4)})),as.character(expression(delta*g^{(4)}))
                               ,as.character(expression(delta*l^{(5)})),as.character(expression(delta*g^{(5)})))
                      ###### theme settings
                      ,axis.label=c(expression(paste("Incremental change ",delta*l^{(2)}," [m]",sep="")
                                               ,paste("Incremental change ",delta*g^{(2)}," [m]",sep="")
                                               ,paste("Incremental change ",delta*l^{(3)}," [m]",sep="")
                                               ,paste("Incremental change ",delta*g^{(3)}," [m]",sep="")
                                               ,paste("Incremental change ",delta*l^{(4)}," [m]",sep="")
                                               ,paste("Incremental change ",delta*g^{(4)}," [m]",sep="")
                                               ,paste("Incremental change ",delta*l^{(5)}," [m]",sep="")
                                               ,paste("Incremental change ",delta*g^{(5)}," [m]",sep=""))))


#### Figure4-b



scatter.Shifts.ggplot(matDistrib = b1b2[,-c(1,2)]
                      ,data.Maxpost = rep(0,8)
                      ,file.path = paste("ScatterPlot_Offsets.pdf",sep="")
                      ###### distribution settings
                      ,prior.param=matrix(c(rep(0,8),c(0.5*sqrt(2),0.5,0.5*sqrt(4),0.5*sqrt(2)
                                                       ,0.5*sqrt(6),0.5*sqrt(3),0.5*sqrt(8),0.5*sqrt(4))),ncol=2)
                      ,loi.type=c(rep(TRUE,8))
                      ,coeff.cor.size=40
                      ,low.cor="black",high.cor="black"
                      ###### densities settings
                      ,coeff.dens=rep(3,8)
                      ,size.gene=2
                      ,size.vline=matrix(2,ncol=2,nrow=8)
                      ,colour.dens=c("#ef8a62","#67a9cf")
                      ,linetype.vline=matrix("longdash",ncol=2,nrow=8)
                      ,wind.lim=matrix(c(rep(-6.1,2),rep(6.1,2)),ncol=2)
                      ###### annotate settings
                      ,sizeDens=rep(40,8)
                      ,coeff.annote=matrix(0.85,ncol=2,nrow=8)
                      ,col.annotate=rep("#737373",8)
                      ,label=c(as.character(expression(b[1]^{(2)})),as.character(expression(b[2]^{(2)}))
                               ,as.character(expression(b[1]^{(3)})),as.character(expression(b[2]^{(3)}))
                               ,as.character(expression(b[1]^{(4)})),as.character(expression(b[2]^{(4)}))
                               ,as.character(expression(b[1]^{(5)})),as.character(expression(b[2]^{(5)})))
                      ###### theme settings
                      ,axis.label=c(expression(paste("Offset ",b[1]^{(2)}," [m]",sep="")
                                               ,paste("Offset ",b[2]^{(2)}," [m]",sep="")
                                               ,paste("Offset ",b[1]^{(3)}," [m]",sep="")
                                               ,paste("Offset ",b[2]^{(3)}," [m]",sep="")
                                               ,paste("Offset ",b[1]^{(4)}," [m]",sep="")
                                               ,paste("Offset ",b[2]^{(4)}," [m]",sep="")
                                               ,paste("Offset ",b[1]^{(5)}," [m]",sep="")
                                               ,paste("Offset ",b[2]^{(5)}," [m]",sep=""))))



