# Clear plots
if(!is.null(dev.list())) dev.off()
# Clear console
cat("\014") 
# Clean workspace
rm(list=ls())
#########################################################################################################
############################################## Settings #################################################
#########################################################################################################
require(ggplot2);require(reshape2);require(scales);require(grid);require(RColorBrewer)


########################################################
########################################################
dire.sim="Meyras/"
study.name="SPD_AllPeriods/"

NameStudy="Meyras_fig5_op1"
op_cooked=FALSE
nperiod=5

nburn=0.5;nslim=10

err.per=0.1;color.inter="#F4A582";color.interBorder="#F4A582";alpha.inter=0.5;color.gaugings="#0571B0"
na.rm=TRUE;shape.gaugings=18;size.gaugings=4;color.RC="#CA0020";size.RC=2;size.wind=20;size.axis=20
face.wind="bold";hydr.op=NULL;color.measurement="#92C5DE";na.rm.meas=TRUE;shape.meas=18;size.meas=4
error.bar.op=NULL;width.errorbar=0.005;time.unit="s";inter.hydr=NULL;blank=FALSE
param.plot=FALSE;color.param.inter="#FEE0B6";color.param.interBorder="#FEE0B6"
alpha.param.inter=0.5;hydr.op=1;time.unit="h";blank=TRUE;alpha.inter=0.5;width.wind=30;height.wind=30
param.plot = TRUE;color.param.inter = "#B2ABD2";color.param.interBorder = "#B2ABD2";

#########################################################################################################
############################################ Data loading ###############################################
#########################################################################################################

if(op_cooked){
  #### Cooked MCMC
  data.MCMC.cooked=as.matrix(read.table(paste(dire.sim,study.name,"Results_MCMC_Cooked.txt",sep="")
                                        ,header=TRUE,dec=".",sep=""))
}else{
  #### Raw MCMC
  data.MCMC.raw=as.matrix(read.table(paste(dire.sim,study.name,"Results_MCMC.txt",sep="")
                                        ,header=TRUE,dec=".",sep=""))
  nsim.tmp=length(data.MCMC.raw[,1])
  data.MCMC.cooked=data.MCMC.raw[seq(nburn*nsim.tmp,nsim.tmp,nslim),]
  rm("data.MCMC.raw")
}

logPost.ncol=which(colnames(data.MCMC.cooked)=="LogPost")

data.MCMC.MaxPost=as.numeric(read.table(paste(dire.sim,study.name,"Results_Summary.txt",sep="")
                                        ,row.names=1,dec=".",sep="",skip = 16))

nsample=length(data.MCMC.cooked[,1])


########################################################################
########### Propagation: Convert MCMC into rating curve results
########################################################################

# grid of stage
hgrid=seq(-1,5.6,0.01)

RC_3controls=function(theta,h,op=1){
  #### Rating curve with discharge error propagation
  Q=0*h
  mask1=which(((h>theta[1])+(h<=theta[13]))==2)
  mask2=which(((h>theta[13])+(h<=theta[7]))==2)
  mask3=which(h>theta[7])
  Q[mask1]=theta[2]*(h[mask1]-theta[1])^theta[3] # section control
  Q[mask2]=theta[5]*(h[mask2]-theta[4])^theta[6] # channel control
  Q[mask3]=theta[5]*(h[mask3]-theta[4])^theta[6] + theta[8]*(h[mask3]-theta[7])^theta[9] # channel + floodway controls
  if(op==1){
    resQ=sapply(Q,function(Q,theta){Q+rnorm(1,mean=0,sd=theta[1]+theta[2]*Q)},theta=c(theta[10],theta[11]))
  }else{
    resQ=Q
  }
  return(resQ)
}

RC_3controls_mp=function(theta,h){
  #### Rating curve for maximum posterior
  Q=0*h
  mask1=which(((h>theta[1])+(h<=theta[12]))==2)
  mask2=which(((h>theta[12])+(h<=theta[7]))==2)
  mask3=which(h>theta[7])
  Q[mask1]=theta[2]*(h[mask1]-theta[1])^theta[3] # section control
  Q[mask2]=theta[5]*(h[mask2]-theta[4])^theta[6] # channel control
  Q[mask3]=theta[5]*(h[mask3]-theta[4])^theta[6] + theta[8]*(h[mask3]-theta[7])^theta[9] # channel + floodway controls
  return(Q)
}

#### Initialisation 
MCMC.save=matrix(NA,nrow=nperiod*nsample,ncol=14) ### b1 a1 c1 b2 a2 c2 b3 a3 c3 gamma1 gamma2 logpost k2 nper
                                                  ### 1  2  3  4  5  6  7  8  9    10     11      12   13  14
MaxPost.save=matrix(NA,nrow=nperiod,ncol=12)### b1 a1 c1 b2 a2 c2 b3 a3 c3 gamma1 gamma2 k2
                                            ### 1  2  3  4  5  6  7  8  9    10     11   12

for(i in 1:nperiod){
  col.num=c(i,6,7,7+i,13:20,6*(i-1)+22)
  MCMC.save[(nsample*(i-1)+1):(i*nsample),]=cbind(data.MCMC.cooked[,col.num],rep(i,nsample))
  col.num.mp=c(i,6,7,7+i,13:19,6*(i-1)+21)
  MaxPost.save[i,]=data.MCMC.MaxPost[col.num.mp]
}

### apply RC for hgrid with structural error

RC.Post=apply(MCMC.save,MARGIN=1,RC_3controls,h=hgrid)

### Maximum posterior computing

RC.MaxPost=apply(MaxPost.save,MARGIN=1,RC_3controls_mp,h=hgrid)

#### Gaugings loading

data.gaug=read.table(paste(dire.sim,"MeyrasGaugings.txt",sep=""),header=TRUE,dec=".",sep="")

#########################################################################################################
############################################## Figure 5 #################################################
#########################################################################################################

### Percentile computing
quants.per1=apply(RC.Post[,1:nsample],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per2=apply(RC.Post[,(nsample+1):(2*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per3=apply(RC.Post[,(2*nsample+1):(3*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per4=apply(RC.Post[,(3*nsample+1):(4*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per5=apply(RC.Post[,(4*nsample+1):(5*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))

data.RC=data.frame(h=rep(hgrid,5),rbind(t(quants.per1),t(quants.per2),t(quants.per3),t(quants.per4),t(quants.per5))
                   ,as.vector(RC.MaxPost),per=rep(1:nperiod,each=length(hgrid)))
colnames(data.RC)=c("h","inf","sup","maxpost","Period")
inter.neg=which(data.RC$inf<0)
data.RC$inf[inter.neg]=0

data.RC$Period=as.factor(data.RC$Period)
data.gaug$Period=as.factor(data.gaug$Period)

plot.RC=ggplot(data.RC[which(data.RC$maxpost>0),])+
  geom_vline(xintercept = MaxPost.save[1,7],colour="black",size=1)+
  geom_vline(xintercept = MaxPost.save[,12],colour=c("#9E0142","#F46D43","#FFFFBF","#ABDDA4","#3288BD"),size=1,linetype="longdash")+
  geom_smooth(aes(x=h,y=maxpost,ymax=sup,ymin=inf,group=Period),size=1,stat='identity',colour="grey",alpha=0.2)+
  geom_path(aes(x=h,y=maxpost,group=Period,colour=Period),na.rm=TRUE,size=1)+
  ### Gaugings
  geom_linerange(aes(x=h,ymax=Q+2*uQ,ymin=Q-2*uQ,colour=Period),data=data.gaug,size=1)+
  geom_point(aes(x=h,y=Q,colour=Period),data=data.gaug,na.rm=na.rm,shape=16,size=size.gaugings)+
  ### Labels
  xlab(expression(paste("Stage h [m]",sep="")))+
  ylab(expression(paste("Discharge Q [",m^3,".",s^-1,"]",sep="")))+
  labs(colour = "Period")+
  scale_colour_manual(values = c("#9E0142","#F46D43","#FFFFBF","#ABDDA4","#3288BD"))+
  scale_y_continuous(breaks=c(0,100,200,300,400,500,600,700,800,900,1000),labels=c(0,100,200,300,400,500,600,700,800,900,1000))+
  coord_cartesian(ylim=c(0.07,550),xlim=c(-0.5,4.8))+
  ### Theme
  theme_bw(base_size=20)+
  theme(axis.text=element_text(size=20),axis.title=element_text(size=30,face="bold")
        ,panel.grid.major=element_line(size=1.2),panel.grid.minor=element_line(size=0.8)
        ,legend.text=element_text(size=20),legend.title=element_text(size=30)
        ,legend.key.size=unit(1.5, "cm"),legend.position="none")
  
## Normal scale
pdf(paste(NameStudy,".pdf",sep=""),16,8,useDingbats=F)
print(plot.RC)
dev.off()

## Logarithmic scale
ylim.wind=c(0.07,550);xlim.wind=c(-0.5,4.8)

pdf(paste(NameStudy,"_Qlog.pdf",sep=""),16,8,useDingbats=F)
print(plot.RC+
        scale_y_log10(breaks=c(0.01,0.1,1,10,100,500),labels=c(0.01,0.1,1,10,100,500)))
dev.off()


#########################################################################################################
############################################## Figure 6 #################################################
#########################################################################################################
##### Loading prior results
# Prior distributions
data.prior=read.table(paste(dire.sim,study.name,"Results_PriorMCSamples.txt",sep="")
                      ,header=TRUE,dec=".",sep="")

##### subplot a

### Posterior results
inter.fig6a=c(6,1,7,13,8,14,16,15,17)
data.boxplot.post.fig6a=data.frame(data.MCMC.cooked[,inter.fig6a])
data.boxplot.prior.fig6a=data.prior[,inter.fig6a]

data.post.notmod=melt(data.boxplot.post.fig6a)
data.prior.notmod=melt(data.boxplot.prior.fig6a)

ntype.post.notmod=length(data.post.notmod$value)
ntype.prior.notmod=length(data.prior.notmod$value)

data.notmod=data.frame(value=c(data.prior.notmod$value,data.post.notmod$value)
                       ,type=factor(c(rep("Prior",ntype.prior.notmod),rep("Post",ntype.post.notmod))
                                    ,levels=c("Prior","Post"))
                       ,Period=factor(c(data.prior.notmod$variable,data.post.notmod$variable)
                                      ,levels=1:9
                                      ,labels=c("a[1]","b[1]^{(1)}","c[1]","a[2]","b[2]^{(1)}"
                                                ,"c[2]","a[3]","b[3]","c[3]")))
############ FIGURE 6-(a)
barplot.paramNotMod=ggplot(data.notmod)+
  geom_boxplot(aes(x=type,y=value,group=type,fill=type),colour="black",outlier.shape = NA,size=1)+
  facet_wrap(~Period,ncol=9,scales="free",labeller = label_parsed)+
  xlab(expression(paste("Period k",sep="")))+
  ylab("Value")+
  # scale_fill_manual(values = c("#f4a582","#0571b0","#92c5de"))+
  scale_fill_manual(values = c("#636363","#ffffff"))+
  # scale_y_continuous(breaks=break.kappa,labels=break.kappa,limits = c(0.5,4))+
  theme_bw(base_size=20)+
  theme(axis.text=element_text(size=30),axis.title=element_text(size=40,face="bold")
        ,panel.grid.major=element_line(size=1.2),panel.grid.minor=element_line(size=0.8)
        ,legend.text=element_text(size=20),legend.title=element_text(size=30)
        ,legend.key.size=unit(1.5, "cm"),legend.position="none"
        ,strip.text.x=element_text(size = 50,face="bold"),axis.title.x=element_blank()
        ,axis.text.x=element_blank(),axis.ticks.x=element_blank()
        ,strip.background = element_rect(fill = c("white","#9E0142","white","white"
                                                  ,"#9E0142","white","white","#9E0142","white")))

pdf(paste("Meyras_fig6a.pdf",sep=""),20,10,useDingbats=F)
print(barplot.paramNotMod)
dev.off()

############ FIGURE 6-(b)

inter.fig6b=as.vector(sapply(1:5,function(x){return(c(x,x+7))}))
data.boxplot.post.fig6b=data.frame(data.MCMC.cooked[,inter.fig6b])
data.boxplot.prior.fig6b=data.prior[,inter.fig6b]

data.post.offset=melt(data.boxplot.post.fig6b)
data.prior.offset=melt(data.boxplot.prior.fig6b)

ntype.post.offset=length(data.post.offset$value)
ntype.prior.offset=length(data.prior.offset$value)

data.incem.mod=data.frame(value=c(data.prior.offset$value,data.post.offset$value)
                          ,type=factor(c(rep("Prior",ntype.prior.offset),rep("Post",ntype.post.offset))
                                       ,levels=c("Prior","Post"))
                          ,Period=factor(c(data.prior.offset$variable,data.post.offset$variable)
                                         ,levels=1:10
                                         ,labels=c("b[1]^{(1)}","b[2]^{(1)}","b[1]^{(2)}","b[2]^{(2)}","b[1]^{(3)}"
                                                   ,"b[2]^{(3)}","b[1]^{(4)}","b[2]^{(4)}"
                                                   ,"b[1]^{(5)}","b[2]^{(5)}")))

break.offset=round(seq(-1,1,0.1),2)

palette=c("#9E0142","#F46D43","#FFFFBF","#ABDDA4","#3288BD")

############ Plot param not mod
barplot.offset=ggplot(data.incem.mod)+
  geom_boxplot(aes(x=type,y=value,group=type,fill=type),colour="black",outlier.shape = NA,size=1)+
  facet_grid(~Period,scales="free",labeller = label_parsed)+
  ylab(expression(paste("Offset [m]",sep="")))+
  # ylab(expression(paste("Transition stage ",kappa^{(k)}," [m]",sep="")))+
  # scale_fill_manual(values = c("#f4a582","#0571b0","#92c5de"))+
  scale_fill_manual(values = c("#636363","#ffffff"))+
  scale_y_continuous(breaks=break.offset,labels=break.offset)+
  coord_cartesian(ylim=c(-0.69,-0.01))+
  theme_bw(base_size=20)+
  theme(axis.text=element_text(size=30),axis.title=element_text(size=40,face="bold")
        ,panel.grid.major=element_line(size=1.2),panel.grid.minor=element_line(size=0.8)
        ,legend.text=element_text(size=20),legend.title=element_text(size=30)
        ,legend.key.size=unit(1.5, "cm"),legend.position="none"
        ,strip.text.x=element_text(size = 50,face="bold"),axis.title.x=element_blank()
        ,axis.text.x=element_blank(),axis.ticks.x=element_blank()
        ,strip.background = element_rect(fill = c("#9E0142","#9E0142","#F46D43","#F46D43"
                                                  ,"#FFFFBF","#FFFFBF","#ABDDA4","#ABDDA4","#3288BD","#3288BD")))

# Generate the ggplot2 plot grob
g <- grid.force(ggplotGrob(barplot.offset))
# Get the names of grobs and their gPaths into a data.frame structure
grobs_df <- do.call(cbind.data.frame, grid.ls(g, print = FALSE))
# Build optimal gPaths that will be later used to identify grobs and edit them
grobs_df$gPath_full <- paste(grobs_df$gPath, grobs_df$name, sep = "::")
grobs_df$gPath_full <- gsub(pattern = "layout::", 
                            replacement = "", 
                            x = grobs_df$gPath_full, 
                            fixed = TRUE)
# Get the gPaths of the strip background grobs
strip_bg_gpath <- grobs_df$gPath_full[grepl(pattern = ".*strip\\.background.*", 
                                            x = grobs_df$gPath_full)]
strip_bg_gpath[1] # example of a gPath for strip background 
## [1] "strip-t-1.7-5-7-5::strip.1-1-1-1::strip.background.x..rect.5091"

# Get the gPaths of the strip titles
strip_txt_gpath <- grobs_df$gPath_full[grepl(pattern = "strip.*titleGrob.*text.*", 
                                             x = grobs_df$gPath_full)]
strip_txt_gpath[1] # example of a gPath for strip title
## [1] "strip-t-1.7-5-7-5::strip.1-1-1-1::GRID.titleGrob.5079::GRID.text.5069"

# Generate some color
n_cols <- length(strip_bg_gpath)
fills <- as.vector(rbind(palette,palette))
# txt_colors <- gray(0:n_cols/n_cols)

# Edit the grobs
for (i in 1:length(strip_bg_gpath)){
  g <- editGrob(grob = g, gPath = strip_bg_gpath[i], gp = gpar(fill = fills[i]))
  # g <- editGrob(grob = g, gPath = strip_txt_gpath[i], gp = gpar(col = txt_colors[i]))
}

require(gridExtra)

pdf(paste("Meyras_fig6b.pdf",sep=""),20,10,useDingbats=F)
print(grid.arrange( g ))
dev.off()

############ FIGURE 6-(c and d) -- global change parameters
###

data.boxplot.post.fig6c=data.frame(matrix(NA,ncol=4,nrow=nsample))
data.boxplot.post.fig6d=data.frame(matrix(NA,ncol=4,nrow=nsample))

nprior=length(data.prior[,1])

data.boxplot.prior.fig6c=data.frame(matrix(NA,ncol=4,nrow=nprior))
data.boxplot.prior.fig6d=data.frame(matrix(NA,ncol=4,nrow=nprior))

inter.glocal=as.vector(sapply(1:5,function(x){return(c(x,x+7))}))
for(i in 1:4){
  data.boxplot.post.fig6c[,i]=data.MCMC.cooked[,inter.glocal[2*i]]-data.MCMC.cooked[,inter.glocal[2*(i+1)]]
  data.boxplot.post.fig6d[,i]=data.MCMC.cooked[,inter.glocal[2*(i-1)+1]]-data.MCMC.cooked[,inter.glocal[2*i+1]]-data.boxplot.post.fig6c[,i]
  data.boxplot.prior.fig6c[,i]=data.prior[,inter.glocal[2*i]]-data.prior[,inter.glocal[2*(i+1)]]
  data.boxplot.prior.fig6d[,i]=data.prior[,inter.glocal[2*(i-1)+1]]-data.prior[,inter.glocal[2*i+1]]-data.boxplot.prior.fig6c[,i]
}


######## Figure 6c
data.post.global=melt(data.boxplot.post.fig6c)
data.prior.global=melt(data.boxplot.prior.fig6c)

ntype.post.global=length(data.post.global$value)
ntype.prior.global=length(data.prior.global$value)

data.incem.mod=data.frame(value=c(data.prior.global$value,data.post.global$value)
                          ,type=factor(c(rep("Prior",ntype.prior.global),rep("Post",ntype.post.global))
                                       ,levels=c("Prior","Post"))
                          ,Period=factor(c(data.prior.global$variable,data.post.global$variable)
                                         ,levels=1:4
                                         ,labels=c("delta*g^{(2)}","delta*g^{(3)}","delta*g^{(4)}","delta*g^{(5)}")))

break.global=round(seq(-1,1,0.1),2)

palette=c("#9E0142","#F46D43","#FFFFBF","#ABDDA4","#3288BD")

############ figure 6c
barplot.global=ggplot(data.incem.mod)+
  geom_boxplot(aes(x=type,y=value,group=type,fill=type),colour="black",outlier.shape = NA,size=1)+
  facet_grid(~Period,scales="free",labeller = label_parsed)+
  ylab(expression(paste("Overall incremental change [m]",sep="")))+
  #####
  scale_fill_manual(values = c("#636363","#ffffff"))+
  scale_y_continuous(breaks=break.global,labels=break.global)+
  coord_cartesian(ylim=c(-0.45,0.45))+
  theme_bw(base_size=20)+
  theme(axis.text=element_text(size=30),axis.title=element_text(size=40,face="bold")
        ,panel.grid.major=element_line(size=1.2),panel.grid.minor=element_line(size=0.8)
        ,legend.text=element_text(size=20),legend.title=element_text(size=30)
        ,legend.key.size=unit(1.5, "cm"),legend.position="none"
        ,strip.text.x=element_text(size = 50,face="bold"),axis.title.x=element_blank()
        ,axis.text.x=element_blank(),axis.ticks.x=element_blank()
        ,strip.background = element_rect(fill = c("#9E0142","#9E0142","#F46D43","#F46D43"
                                                  ,"#FFFFBF","#FFFFBF","#ABDDA4","#ABDDA4","#3288BD","#3288BD")))

# Generate the ggplot2 plot grob
g <- grid.force(ggplotGrob(barplot.global))
# Get the names of grobs and their gPaths into a data.frame structure
grobs_df <- do.call(cbind.data.frame, grid.ls(g, print = FALSE))
# Build optimal gPaths that will be later used to identify grobs and edit them
grobs_df$gPath_full <- paste(grobs_df$gPath, grobs_df$name, sep = "::")
grobs_df$gPath_full <- gsub(pattern = "layout::", 
                            replacement = "", 
                            x = grobs_df$gPath_full, 
                            fixed = TRUE)
# Get the gPaths of the strip background grobs
strip_bg_gpath <- grobs_df$gPath_full[grepl(pattern = ".*strip\\.background.*", 
                                            x = grobs_df$gPath_full)]
strip_bg_gpath[1] # example of a gPath for strip background 
## [1] "strip-t-1.7-5-7-5::strip.1-1-1-1::strip.background.x..rect.5091"

# Get the gPaths of the strip titles
strip_txt_gpath <- grobs_df$gPath_full[grepl(pattern = "strip.*titleGrob.*text.*", 
                                             x = grobs_df$gPath_full)]
strip_txt_gpath[1] # example of a gPath for strip title
## [1] "strip-t-1.7-5-7-5::strip.1-1-1-1::GRID.titleGrob.5079::GRID.text.5069"

# Generate some color
n_cols <- length(strip_bg_gpath)
fills <- as.vector(palette[-1])
# txt_colors <- gray(0:n_cols/n_cols)

# Edit the grobs
for (i in 1:length(strip_bg_gpath)){
  g <- editGrob(grob = g, gPath = strip_bg_gpath[i], gp = gpar(fill = fills[i]))
  # g <- editGrob(grob = g, gPath = strip_txt_gpath[i], gp = gpar(col = txt_colors[i]))
}

require(gridExtra)

pdf(paste("Meyras_fig6c.pdf",sep=""),20,10,useDingbats=F)
print(grid.arrange( g ))
dev.off()

######## Figure 6d
data.post.local=melt(data.boxplot.post.fig6d)
data.prior.local=melt(data.boxplot.prior.fig6d)

ntype.post.local=length(data.post.local$value)
ntype.prior.local=length(data.prior.local$value)

data.incem.mod=data.frame(value=c(data.prior.local$value,data.post.local$value)
                          ,type=factor(c(rep("Prior",ntype.prior.local),rep("Post",ntype.post.local))
                                       ,levels=c("Prior","Post"))
                          ,Period=factor(c(data.prior.local$variable,data.post.local$variable)
                                         ,levels=1:4
                                         ,labels=c("delta*l^{(2)}","delta*l^{(3)}","delta*l^{(4)}","delta*l^{(5)}")))

break.local=round(seq(-1,1,0.1),2)

palette=c("#9E0142","#F46D43","#FFFFBF","#ABDDA4","#3288BD")

############ figure 6d
barplot.local=ggplot(data.incem.mod)+
  geom_boxplot(aes(x=type,y=value,group=type,fill=type),colour="black",outlier.shape = NA,size=1)+
  facet_grid(~Period,scales="free",labeller = label_parsed)+
  ylab(expression(paste("Local incremental change [m]",sep="")))+
  #####
scale_fill_manual(values = c("#636363","#ffffff"))+
  scale_y_continuous(breaks=break.local,labels=break.local)+
  coord_cartesian(ylim=c(-0.45,0.45))+
  theme_bw(base_size=20)+
  theme(axis.text=element_text(size=30),axis.title=element_text(size=40,face="bold")
        ,panel.grid.major=element_line(size=1.2),panel.grid.minor=element_line(size=0.8)
        ,legend.text=element_text(size=20),legend.title=element_text(size=30)
        ,legend.key.size=unit(1.5, "cm"),legend.position="none"
        ,strip.text.x=element_text(size = 50,face="bold"),axis.title.x=element_blank()
        ,axis.text.x=element_blank(),axis.ticks.x=element_blank()
        ,strip.background = element_rect(fill = c("#9E0142","#9E0142","#F46D43","#F46D43"
                                                  ,"#FFFFBF","#FFFFBF","#ABDDA4","#ABDDA4","#3288BD","#3288BD")))

# Generate the ggplot2 plot grob
g <- grid.force(ggplotGrob(barplot.local))
# Get the names of grobs and their gPaths into a data.frame structure
grobs_df <- do.call(cbind.data.frame, grid.ls(g, print = FALSE))
# Build optimal gPaths that will be later used to identify grobs and edit them
grobs_df$gPath_full <- paste(grobs_df$gPath, grobs_df$name, sep = "::")
grobs_df$gPath_full <- gsub(pattern = "layout::", 
                            replacement = "", 
                            x = grobs_df$gPath_full, 
                            fixed = TRUE)
# Get the gPaths of the strip background grobs
strip_bg_gpath <- grobs_df$gPath_full[grepl(pattern = ".*strip\\.background.*", 
                                            x = grobs_df$gPath_full)]
strip_bg_gpath[1] # example of a gPath for strip background 
## [1] "strip-t-1.7-5-7-5::strip.1-1-1-1::strip.background.x..rect.5091"

# Get the gPaths of the strip titles
strip_txt_gpath <- grobs_df$gPath_full[grepl(pattern = "strip.*titleGrob.*text.*", 
                                             x = grobs_df$gPath_full)]
strip_txt_gpath[1] # example of a gPath for strip title
## [1] "strip-t-1.7-5-7-5::strip.1-1-1-1::GRID.titleGrob.5079::GRID.text.5069"

# Generate some color
n_cols <- length(strip_bg_gpath)
fills <- as.vector(palette[-1])
# txt_colors <- gray(0:n_cols/n_cols)

# Edit the grobs
for (i in 1:length(strip_bg_gpath)){
  g <- editGrob(grob = g, gPath = strip_bg_gpath[i], gp = gpar(fill = fills[i]))
  # g <- editGrob(grob = g, gPath = strip_txt_gpath[i], gp = gpar(col = txt_colors[i]))
}

require(gridExtra)

pdf(paste("Meyras_fig6d.pdf",sep=""),20,10,useDingbats=F)
print(grid.arrange( g ))
dev.off()