# Clear plots
if(!is.null(dev.list())) dev.off()
# Clear console
cat("\014") 
# Clean workspace
rm(list=ls())
#########################################################################################################
############################################## Settings #################################################
#########################################################################################################
require(ggplot2);require(reshape2);require(scales);require(grid);require(RColorBrewer)


########################################################
########################################################
dire.sim="Wairau/"
study.name="SPD_AllPeriods/"

NameStudy="Wairau"
op_cooked=FALSE
nperiod=60

nburn=0.5;nslim=10

#########################################################################################################
############################################# Functions #################################################
#########################################################################################################

CountPeriod=function(file,ncontrol){
  tab.tmp=read.table(file,header=TRUE,dec=".",sep="",nrows=1)
}

#########################################################################################################
############################################ Data loading ###############################################
#########################################################################################################

if(op_cooked){
  #### Cooked MCMC
  data.MCMC.cooked=as.matrix(read.table(paste(dire.sim,study.name,"Results_MCMC_Cooked.txt",sep="")
                                        ,header=TRUE,dec=".",sep=""))
}else{
  #### Raw MCMC
  data.MCMC.raw=as.matrix(read.table(paste(dire.sim,study.name,"Results_MCMC.txt",sep="")
                                        ,header=TRUE,dec=".",sep=""))
  nsim.tmp=length(data.MCMC.raw[,1])
  data.MCMC.cooked=data.MCMC.raw[seq(nburn*nsim.tmp,nsim.tmp,nslim),]
  rm("data.MCMC.raw")
}

logPost.ncol=which(colnames(data.MCMC.cooked)=="LogPost")

data.MCMC.MaxPost=as.numeric(read.table(paste(dire.sim,study.name,"Results_Summary.txt",sep="")
                                        ,row.names=1,dec=".",sep="",skip = 16))

nsample=length(data.MCMC.cooked[,1])


#### RC propagation

hgrid=seq(1,7,0.01)

RC_2controls=function(theta,h,op=1){
  Q=0*h
  mask1=which(((h>theta[1])+(h<=theta[10]))==2)
  mask2=which(h>theta[10])
  Q[mask1]=theta[2]*(h[mask1]-theta[1])^theta[3] # section control
  Q[mask2]=theta[5]*(h[mask2]-theta[4])^theta[6] # Channel controls
  if(op==1){
    resQ=sapply(Q,function(Q,theta){Q+rnorm(1,mean=0,sd=theta[1]+theta[2]*Q)},theta=c(theta[7],theta[8]))
  }else{
    resQ=Q
  }
  return(resQ)
}

RC_2controls_mp=function(theta,h){
  Q=0*h
  mask1=which(((h>theta[1])+(h<=theta[9]))==2)
  mask2=which(h>theta[9])
  Q[mask1]=theta[2]*(h[mask1]-theta[1])^theta[3] # section control
  Q[mask2]=theta[5]*(h[mask2]-theta[4])^theta[6] # Channel controls
  return(Q)
}
## Convert MCMC 

MCMC.save=matrix(NA,nrow=nperiod*nsample,ncol=11) ### b1 a1 c1 b2 a2 c2 gamma1 gamma2 logpost k2 nper
                                                  ### 1  2  3  4  5  6     7     8       9    10  11
MaxPost.save=matrix(NA,nrow=nperiod,ncol=10)### b1 a1 c1 b2 a2 c2 gamma1 gamma2 k2 nper
                                            ### 1  2  3  4  5  6     7     8    9   10

for(i in 1:nperiod){
  col.num=c(i,nperiod+i,(2*nperiod+1):(2*nperiod+7),2*nperiod+4*(i-1)+9)
  MCMC.save[(nsample*(i-1)+1):(i*nsample),]=cbind(data.MCMC.cooked[,col.num],rep(i,nsample))
  col.num.mp=c(i,nperiod+i,(2*nperiod+1):(2*nperiod+6),2*nperiod+4*(i-1)+8)
  MaxPost.save[i,]=c(data.MCMC.MaxPost[col.num.mp],i)
}

### apply RC for hgrid with structural error

RC.Post=apply(MCMC.save,MARGIN=1,RC_2controls,h=hgrid)


RC.MaxPost=apply(MaxPost.save,MARGIN=1,RC_2controls_mp,h=hgrid)
  
data.gaug=read.table(paste(dire.sim,"WairauGaugings.txt",sep=""),header=TRUE,dec=".",sep="")

data.gaug=data.gaug[-which(data.gaug$Q==-9999),]

#########################################################################################################
############################################## Figure 5 #################################################
#########################################################################################################
List.RC.quants=list(NULL)

for(i in 1:nperiod){
  data.tmp=apply(RC.Post[,(nsample*(i-1)+1):(nsample*i)],MARGIN=1,quantile,probs=c(0.025,0.975))
  data.tmp=apply(data.tmp,MARGIN=c(1,2),function(x){ifelse(x<0,0,x)})
  List.RC.quants[[i]]=data.frame(cbind(hgrid,t(data.tmp),RC.MaxPost[,i]))
  colnames(List.RC.quants[[i]])=c("h","inf","sup","maxpost")
}


inter.per=c(2,25,29,38,39,51,52,53,54)
# inter.per=c(25:35)
require(RColorBrewer)
nRC=length(inter.per)
palette.per=brewer.pal(nRC,"Spectral")

# table(data.gaug$period)


data.plot.RC=data.frame(do.call("rbind", List.RC.quants[inter.per]),Period=rep(inter.per,each=length(hgrid)))

inter.null=which(data.plot.RC$maxpost==0)



err.per=0.1;color.inter="#F4A582";color.interBorder="#F4A582";alpha.inter=0.5;color.gaugings="#0571B0"
na.rm=TRUE;shape.gaugings=18;size.gaugings=4;color.RC="#CA0020";size.RC=2;size.wind=20;size.axis=20
face.wind="bold";hydr.op=NULL;color.measurement="#92C5DE";na.rm.meas=TRUE;shape.meas=18;size.meas=4
error.bar.op=NULL;width.errorbar=0.005;time.unit="s";inter.hydr=NULL;blank=FALSE;width.wind=20
height.wind=20;param.plot=FALSE;color.param.inter="#FEE0B6";color.param.interBorder="#FEE0B6"
alpha.param.inter=0.5;hydr.op=1;time.unit="h";blank=TRUE;alpha.inter=0.5;width.wind=40;height.wind=20
param.plot = TRUE;color.param.inter = "#B2ABD2";color.param.interBorder = "#B2ABD2";

vect.break=seq(1,9,1)
vect.emp=rep("",8)
breaks.wind=c(vect.break*0.01,vect.break*0.1,vect.break,vect.break*10,vect.break*100,vect.break*1000)
labels.wind=c(0.01,vect.emp,0.1,vect.emp,1,vect.emp,10,vect.emp,100,vect.emp,1000,vect.emp)
ylim.wind=c(3.5,3900)
xlim.wind=c(1.8,6.5)

pos.num=function(x.int){inter=which(x.int==data.gaug$period);return(inter)}

inter.gaug=unlist(sapply(inter.per,pos.num), recursive = TRUE, use.names = TRUE)
data.obs=data.gaug[inter.gaug,]
data.obs$period=factor(data.obs$period)

data.RC=data.plot.RC #[-inter.null,]
data.RC$Period=factor(data.RC$Period)

plot.RC=ggplot(data.RC)+
  geom_vline(xintercept = MaxPost.save[inter.per,9],colour=palette.per,size=1,linetype="longdash")+
  geom_smooth(aes(x=h,y=maxpost,ymax=sup,ymin=inf,group=Period),size=1,stat='identity',colour="grey",alpha=0.2)+
  geom_path(aes(x=h,y=maxpost,group=Period,colour=Period),na.rm=TRUE,size=1)+
  ### Gaugings
  geom_linerange(aes(x=h,ymax=Q+2*uQ,ymin=Q-2*uQ,colour=period),data=data.obs,size=1)+
  geom_point(aes(x=h,y=Q,colour=period),data=data.obs,na.rm=na.rm,shape=16,size=size.gaugings)+
  ### Labels
  xlab(expression(paste("Stage h [m]",sep="")))+
  ylab(expression(paste("Discharge Q [",m^3,".",s^-1,"]",sep="")))+
  labs(colour = "Period")+
  scale_colour_manual(values = palette.per)+
  # scale_y_continuous(breaks=breaks.wind,labels=labels.wind)+
  coord_cartesian(ylim=ylim.wind,xlim=xlim.wind)+
  ### Theme
  theme_bw(base_size=20)+
  theme(axis.text=element_text(size=20),axis.title=element_text(size=30,face="bold")
        ,panel.grid.major=element_line(size=1.2),panel.grid.minor=element_line(size=0.8)
        ,legend.text=element_text(size=20),legend.title=element_text(size=30)
        ,legend.key.size=unit(1.5, "cm"),legend.position="none")
  

pdf(paste(NameStudy,"fig7.pdf",sep=""),16,8,useDingbats=F)
print(plot.RC)
dev.off()



pdf(paste(NameStudy,"fig7_Qlog.pdf",sep=""),16,8,useDingbats=F)
print(plot.RC+
        scale_y_log10(breaks=breaks.wind,labels=labels.wind)+
        theme(panel.grid.minor=element_blank()))
dev.off()

