require(ggplot2);require(gridExtra)
#-----------------------------------------------------
# read files and define parameters
#-----------------------------------------------------
mcmcFile='Wairau/SPD_AllPeriods/Results_MCMC_Cooked.txt'
residualFile='Wairau/SPD_AllPeriods/Results_Residuals.txt'
nperiod=60
# MCMC file
mcmc=read.table(mcmcFile,header=T)
res=read.table(residualFile,header=T)
indx=1:nrow(mcmc)
n=length(indx)
# Gaugings
HQ=read.table('Wairau/WairauGaugings.txt',header=T)
# define parameters
b1=mcmc[indx,1:nperiod]
a1=mcmc[indx,nperiod+(1:nperiod)]
c1=mcmc[indx,2*nperiod+1]
b2=mcmc[indx,2*nperiod+2]
a2=mcmc[indx,2*nperiod+3]
c2=mcmc[indx,2*nperiod+4]
gamma1=mcmc[indx,2*nperiod+5]
gamma2=mcmc[indx,2*nperiod+6]
logpost=mcmc[indx,2*nperiod+7]
ks=mcmc[indx,(2*nperiod+8):NCOL(mcmc)]
k2=ks[,seq(2,NCOL(ks),4)]
# find maxpost
imax=which.max(logpost)
maxpost=as.numeric(mcmc[indx[imax],])

#-----------------------------------------------------
# Residuals
#-----------------------------------------------------
mask=res$Y1_stdres!=-9999
DF=data.frame(res=res$Y1_stdres[mask],indx=1:sum(mask),
              period=HQ$period[mask],Qsim=res$Y1_sim[mask])
g1=ggplot(DF,aes(x=indx,y=res))
g1=g1+geom_point(aes(fill=period),pch=21)
g1=g1+scale_fill_gradientn(colours=rainbow(nperiod))
g1=g1+labs(y='standardized residual [/]',x='index')
g1=g1+theme_bw()+theme(legend.position="top")

g2=ggplot(DF,aes(x=Qsim,y=res))
g2=g2+geom_point(aes(fill=period),pch=21)
g2=g2+scale_fill_gradientn(colours=rainbow(nperiod))
g2=g2+scale_x_continuous(trans='log10')
g2=g2+labs(y='standardized residual [/]',x=expression(paste("rating curve discharge [", m^{3},'.',s^{-1}, "]")))
g2=g2+theme_bw()+theme(legend.position="none")

pdf('Residuals.pdf',width=8,height=6,useDingbats=F)
grid.arrange(grobs=list(g1,g2),layout_matrix=rbind(1,1,1,1,1,2,2,2,2))
dev.off()
#-----------------------------------------------------
# boxplot of a1/b1
#-----------------------------------------------------
# define customized boxplot function 
myboxplot <- function(x) {
  r <- quantile(x, probs = c(0.025, 0.25, 0.5, 0.75, 0.975))
  names(r) <- c("ymin", "lower", "middle", "upper", "ymax")
  r
}
# create data frame 
DF=data.frame(period=vector(mode='integer'),
              nGaugings=vector(mode='integer'),
              param=vector(mode='character'),
              val=vector(mode='double'))
for (i in 1:nperiod){
  DF=rbind(DF,data.frame(
    period=rep(i,4*n),
    nGaugings=rep(sum(HQ$period==i),4*n),
    param=c(rep('a[1] ~~ ~bgroup("[",m^{4/3}%.%s^{-1},"]")',n),
            rep('b[1] == kappa[1] ~~ ~bgroup("[",m,"]")',n),
            rep('kappa[2] ~~ ~bgroup("[",m,"]")',n),
            rep('k2-k1',n)),
    val=c(a1[,i],b1[,i],k2[,i],k2[,i]-b1[,i])
    ))
}
# create ggplot
g=ggplot(DF[DF$param!='k2-k1',],aes(period,val))
g=g+stat_summary(fun.data=myboxplot,geom="boxplot",aes(fill=nGaugings),lwd=0)
g=g+scale_fill_gradient('Number of available gaugings',low='white',high='red')
g=g+facet_wrap(~param,scales='free_y',ncol=1,labeller = label_parsed)
g=g+labs(y='parameter value',x='period')
g=g+theme_bw()
g=g+theme(legend.position="top",
          strip.text.x=element_text(size=10,face='bold'))
pdf('a1b1.pdf',width=6,height=6,useDingbats=F)
print(g)
dev.off()
#-----------------------------------------------------
# RC movie
#-----------------------------------------------------
# define rating curve function
RC<-function(h,theta,k2){
  Q=0*h
  mask=h<k2
  Q[mask]=theta[2]*(h[mask]-theta[1])^theta[3] # low-flow channel
  Q[!mask]=theta[5]*(h[!mask]-theta[4])^theta[6] # high-flow channel
  return(Q)
}
# create backgroung plot (all gaugings in light gray)
g0=ggplot(HQ,aes(h,Q))
g0=g0+geom_point(color='lightgray')
g0=g0+theme_bw()+labs(x='h [m]',y=expression(paste("Q [", m^{3},'.',s^{-1}, "]")))
# Make one plot per period
k=0 # counter
hmax=8 # upper end of RC range of validity
Qmax=5000 # y-axis limit
hgrid=seq(min(b1[imax,]),hmax,,400)
mov <- vector("list",nperiod)
for (i in 1:nperiod){
  message(paste('Period',i,'/',nperiod))
  mask=(HQ$period==i)
  if(sum(mask>0)){ # if 0 gaugings in period i, BaM does not compute k2 for this period and the RC cannot directly be computed
    k=k+1
    Qs0=matrix(0,n,length(hgrid))
    Qs=matrix(0,n,length(hgrid))
    for(j in 1:n){
      # extract relevant parameters
      theta=c(b1[j,i],a1[j,i],c1[j],b2[j],a2[j],c2[j])
      gamma=c(gamma1[j],gamma2[j])
      transition=k2[j,k]
      # rating curve 
      Qs0[j,]=RC(hgrid,theta,transition)
      # add structural errors
      Qs[j,]=Qs0[j,]+rnorm(length(hgrid),mean=0,sd=gamma[1]+gamma[2]*Qs0[j,])
    }
    # compute maxpost RC and uncertainty interval
    UI=apply(Qs,2,quantile,c(0.025,0.975),na.rm=T)
    maxpost=Qs0[imax,]
    # make ggplot
    df=data.frame(h=hgrid,Q=maxpost)
    U.RC=data.frame(
      x=c(hgrid,hgrid[seq(length(hgrid),1,-1)]),
      y=c(UI[1,],UI[2,seq(length(hgrid),1,-1)])
      )
    U.k1=data.frame(
      x=quantile(b1[,i],c(0.025,0.975,0.975,0.025)),
      y=c(0,0,2*Qmax,2*Qmax)
    )
    U.k2=data.frame(
      x=quantile(k2[,k],c(0.025,0.975,0.975,0.025)),
      y=c(0,0,2*Qmax,2*Qmax)
    )
    # activation stages and their uncertainty
    g=g0+geom_polygon(data=U.k1,aes(x=x,y=y),fill='green',alpha=0.1)  
    g=g+geom_polygon(data=U.k2,aes(x=x,y=y),fill='green',alpha=0.1)  
    # RC and its uncertainty
    g=g+geom_polygon(data=U.RC,aes(x=x,y=y),fill='red',alpha=0.25)
    g=g+geom_line(data=df,aes(x=h,y=Q),color='red')
    # gaugings for current period
    g=g+geom_point(data=HQ[mask,],aes(x=h,y=Q),color='darkblue')
    # theme labs etc.
    g=g+labs(title=paste('Period',i))
    g=g+theme(plot.title=element_text(hjust=0.5))
    g=g+scale_y_continuous(trans='log10')
    g=g+coord_cartesian(ylim=c(1,Qmax),xlim=c(hgrid[1],hmax))
    mov[[k]] <- g
  }
}
# save all ggplots as images
if(!dir.exists('img')){dir.create('img',recursive=T)}
resol=200;size=75;fmt=3/2
for(i in 1:k){
  file=paste('img/RC',formatC(i,width=nchar(as.character(k)),flag="0"),'.jpg',sep='')
  jpeg(file,width=fmt*size,height=size,units='mm',res=resol)
  print(mov[[i]])
  dev.off()
}
