# Clear plots
if(!is.null(dev.list())) dev.off()
# Clear console
cat("\014") 
# Clean workspace
rm(list=ls())
#########################################################################################################
############################################## Settings #################################################
#########################################################################################################
require(ggplot2);require(reshape2);require(scales);require(grid);require(RColorBrewer)


########################################################
########################################################
dire.sim="Meyras/"
study.name="SPD_Period1/"

NameStudy="Meyras_fig7"
op_cooked=FALSE
nperiod=5

nburn=0.5;nslim=10

err.per=0.1;color.inter="#F4A582";color.interBorder="#F4A582";alpha.inter=0.5;color.gaugings="#0571B0"
na.rm=TRUE;shape.gaugings=18;size.gaugings=4;color.RC="#CA0020";size.RC=2;size.wind=20;size.axis=20
face.wind="bold";hydr.op=NULL;color.measurement="#92C5DE";na.rm.meas=TRUE;shape.meas=18;size.meas=4
error.bar.op=NULL;width.errorbar=0.005;time.unit="s";inter.hydr=NULL;blank=FALSE
param.plot=FALSE;color.param.inter="#FEE0B6";color.param.interBorder="#FEE0B6"
alpha.param.inter=0.5;hydr.op=1;time.unit="h";blank=TRUE;alpha.inter=0.5;width.wind=30;height.wind=30
param.plot = TRUE;color.param.inter = "#B2ABD2";color.param.interBorder = "#B2ABD2";

#########################################################################################################
############################################ Data loading ###############################################
#########################################################################################################

if(op_cooked){
  #### Cooked MCMC
  data.MCMC.cooked=as.matrix(read.table(paste(dire.sim,study.name,"Results_MCMC_Cooked.txt",sep="")
                                        ,header=TRUE,dec=".",sep=""))
}else{
  #### Raw MCMC
  data.MCMC.raw=as.matrix(read.table(paste(dire.sim,study.name,"Results_MCMC.txt",sep="")
                                        ,header=TRUE,dec=".",sep=""))
  nsim.tmp=length(data.MCMC.raw[,1])
  data.MCMC.cooked=data.MCMC.raw[seq(nburn*nsim.tmp,nsim.tmp,nslim),]
  rm("data.MCMC.raw")
}

logPost.ncol=which(colnames(data.MCMC.cooked)=="LogPost")

data.MCMC.MaxPost=as.numeric(read.table(paste(dire.sim,study.name,"Results_Summary.txt",sep="")
                                        ,row.names=1,dec=".",sep="",skip = 16))

nsample=length(data.MCMC.cooked[,1])


#### RC propagation

hgrid=seq(-1,5.6,0.01)

RC_3controls=function(theta,h,op=1){
  Q=0*h
  mask1=which(((h>theta[1])+(h<=theta[13]))==2)
  mask2=which(((h>theta[13])+(h<=theta[7]))==2)
  mask3=which(h>theta[7])
  Q[mask1]=theta[2]*(h[mask1]-theta[1])^theta[3] # section control
  Q[mask2]=theta[5]*(h[mask2]-theta[4])^theta[6] # channel control
  Q[mask3]=theta[5]*(h[mask3]-theta[4])^theta[6] + theta[8]*(h[mask3]-theta[7])^theta[9] # channel + floodway controls
  if(op==1){
    resQ=sapply(Q,function(Q,theta){Q+rnorm(1,mean=0,sd=theta[1]+theta[2]*Q)},theta=c(theta[10],theta[11]))
  }else{
    resQ=Q
  }
  return(resQ)
}

RC_3controls_mp=function(theta,h){
  Q=0*h
  mask1=which(((h>theta[1])+(h<=theta[12]))==2)
  mask2=which(((h>theta[12])+(h<=theta[7]))==2)
  mask3=which(h>theta[7])
  Q[mask1]=theta[2]*(h[mask1]-theta[1])^theta[3] # section control
  Q[mask2]=theta[5]*(h[mask2]-theta[4])^theta[6] # channel control
  Q[mask3]=theta[5]*(h[mask3]-theta[4])^theta[6] + theta[8]*(h[mask3]-theta[7])^theta[9] # channel + floodway controls
  return(Q)
}
## Convert MCMC into a "melt" data.frame

MCMC.save=matrix(NA,nrow=nperiod*nsample,ncol=14) ### b1 a1 c1 b2 a2 c2 b3 a3 c3 gamma1 gamma2 logpost k2 nper
                                                  ### 1  2  3  4  5  6  7  8  9    10     11      12   13  14
MaxPost.save=matrix(NA,nrow=nperiod,ncol=12)### b1 a1 c1 b2 a2 c2 b3 a3 c3 gamma1 gamma2 k2
                                            ### 1  2  3  4  5  6  7  8  9    10     11   12
#####
# b1_1 b1_2 b1_3 b1_4 b1_5 a1 c1 b2_1 b2_2 b2_3 b2_4 b2_5 a2 c2 
#  1    2    3    4    5    6  7  8    9    10   11   12  13 14
# b3 a3 c3 Y1_gamma1 Y1_gamma2 LogPost k1_1 k2_1 k3_1 ktype1_1 ktype2_1 ktype3_1
# 15 16 17     18        19       20    21   22   23      24       25      26

for(i in 1:nperiod){
  col.num=c(i,6,7,7+i,13:20,6*(i-1)+22)
  MCMC.save[(nsample*(i-1)+1):(i*nsample),]=cbind(data.MCMC.cooked[,col.num],rep(i,nsample))
  col.num.mp=c(i,6,7,7+i,13:19,6*(i-1)+21)
  MaxPost.save[i,]=data.MCMC.MaxPost[col.num.mp]
}

### apply RC for hgrid with structural error

RC.Post=apply(MCMC.save,MARGIN=1,RC_3controls,h=hgrid)


RC.MaxPost=apply(MaxPost.save,MARGIN=1,RC_3controls_mp,h=hgrid)
  
data.gaug=read.table(paste(dire.sim,"MeyrasGaugings.txt",sep=""),header=TRUE,dec=".",sep="")

#########################################################################################################
############################################## Figure 9 #################################################
#########################################################################################################

##### Loading prior results
# Prior distributions
# data.prior=read.table(paste(dire.sim,study.name,"Results_PriorMCSamples_fig7.txt",sep="")
#                       ,header=TRUE,dec=".",sep="")

# require(rootSolve)
# # Transition computing
# Trans.func=function(x,theta){
#   theta[2]*((x-theta[1])^theta[3])-theta[5]*((x-theta[4])^theta[6])
# }
# 
# kappa_compute=function(x,nper){
#   return.vect=NULL
#   vect.rep=c(0,3,0)
#   for(i in 1:nper){
#     inter.per.tmp=c(i,nper+1,nper+2,nper+2+i,(2*nper+3):(2*nper+7))
#     theta.tmp=x[inter.per.tmp]
#     
#     ## Newton-R computing
#     min.inter=max(as.numeric(theta.tmp[1],theta.tmp[4]))
#     interval= c(min.inter,as.numeric(x[2*nper+5]))
#     x.root=uniroot.all(Trans.func, interval, lower = min(interval), upper = max(interval),
#                   tol = .Machine$double.eps^0.2, maxiter = 1000, n = 1000,theta=as.numeric(theta.tmp))
#     if(length(x.root)==0){
#       return.vect=c(return.vect,theta.tmp[1],NA,as.numeric(x[2*nper+5]),vect.rep)
#     }else if(length(x.root)==1){
#       return.vect=c(return.vect,theta.tmp[1],x.root,as.numeric(x[2*nper+5]),vect.rep)
#     }else{
#       return.vect=c(return.vect,theta.tmp[1],x.root[2],as.numeric(x[2*nper+5]),vect.rep)
#     }
#   }
#   return(as.numeric(return.vect))
# }


# Prior distributions
data.prior=read.table(paste(dire.sim,"SPD_Period1_0gaugings/Results_MCMC.txt",sep="")
                      ,header=TRUE,dec=".",sep="")

nsim.prior=length(data.prior[-1,1])
data.prior.slim=as.matrix(data.prior[seq(0.5*nsim.prior,nsim.prior+1,10),])

prior.mean=apply(data.prior.slim,MARGIN=2,mean,na.rm=TRUE)

nsample.prior=length(data.prior.slim[,1])
MCMC.prior.save=matrix(NA,nrow=nperiod*nsample.prior,ncol=14) ### b1 a1 c1 b2 a2 c2 b3 a3 c3 gamma1 gamma2 logpost k2 nper
### 1  2  3  4  5  6  7  8  9    10     11      12   13  14
#####
# b1_1 b1_2 b1_3 b1_4 b1_5 a1 c1 b2_1 b2_2 b2_3 b2_4 b2_5 a2 c2 
#  1    2    3    4    5    6  7  8    9    10   11   12  13 14
# b3 a3 c3 Y1_gamma1 Y1_gamma2 LogPost k1_1 k2_1 k3_1 ktype1_1 ktype2_1 ktype3_1
# 15 16 17     18        19       20    21   22   23      24       25      26

for(i in 1:nperiod){
  col.num=c(i,6,7,7+i,13:20,6*(i-1)+22)
  MCMC.prior.save[(nsample.prior*(i-1)+1):(i*nsample.prior),]=cbind(data.prior.slim[,col.num],rep(i,nsample.prior))
}

### apply RC for hgrid with structural error

RC.Prior=apply(MCMC.prior.save,MARGIN=1,RC_3controls,h=hgrid,op=0)

quants.prior.per1=apply(RC.Prior[,1:nsample.prior],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.prior.per2=apply(RC.Prior[,(nsample.prior+1):(2*nsample.prior)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.prior.per3=apply(RC.Prior[,(2*nsample.prior+1):(3*nsample.prior)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.prior.per4=apply(RC.Prior[,(3*nsample.prior+1):(4*nsample.prior)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.prior.per5=apply(RC.Prior[,(4*nsample.prior+1):(5*nsample.prior)],MARGIN=1,quantile,probs=c(0.025,0.975))

data.RC.prior=data.frame(h=rep(hgrid,5),rbind(t(quants.prior.per1),t(quants.prior.per2),t(quants.prior.per3),t(quants.prior.per4),t(quants.prior.per5))
                   ,per=rep(1:nperiod,each=length(hgrid)))
colnames(data.RC.prior)=c("h","inf","sup","Period")
inter.neg=which(data.RC.prior$inf<0)
data.RC.prior$inf[inter.neg]=0




quants.per1=apply(RC.Post[,1:nsample],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per2=apply(RC.Post[,(nsample+1):(2*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per3=apply(RC.Post[,(2*nsample+1):(3*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per4=apply(RC.Post[,(3*nsample+1):(4*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))
quants.per5=apply(RC.Post[,(4*nsample+1):(5*nsample)],MARGIN=1,quantile,probs=c(0.025,0.975))

data.RC=data.frame(h=rep(hgrid,5),rbind(t(quants.per1),t(quants.per2),t(quants.per3),t(quants.per4),t(quants.per5))
                   ,as.vector(RC.MaxPost),per=rep(1:nperiod,each=length(hgrid)))
colnames(data.RC)=c("h","inf","sup","maxpost","Period")
inter.neg=which(data.RC$inf<0)
data.RC$inf[inter.neg]=0

data.RC$Period=as.factor(data.RC$Period)
data.gaug$Period=as.factor(data.gaug$Period)


palette.col=c("#9E0142","#F46D43","#FFFFBF","#ABDDA4","#3288BD")
breaks.log=c(0.01,0.1,1,10,100,500,1000,5000)
labels.log=c(0.01,0.1,1,10,100,500,1000,5000)
breaks=c(0,50,100,200,300,400,500)
labels=c(0,50,100,200,300,400,500)

ylim.wind=c(0.07,1000)
xlim.wind=c(-0.7,4.8)

ylim.wind.norm=c(0,320)
xlim.wind.norm=c(-0.6,1.95)

nper1=1
inter.Gaug.per1=which(data.gaug$Period==nper1)
RC_per1=ggplot(data.RC[which(data.RC$Period==nper1),])+
  geom_vline(xintercept = MaxPost.save[nper1,7],colour="black",size=1)+
  geom_vline(xintercept = MaxPost.save[nper1,12],colour="black",size=1,linetype="longdash")+
  geom_ribbon(aes(x=h,ymax=sup,ymin=inf,group=Period),data=data.RC.prior[which(data.RC.prior$Period==nper1),]
              ,size=1,stat='identity',fill="#67a9cf",colour="#67a9cf",alpha=0.5)+
  geom_ribbon(aes(x=h,ymax=sup,ymin=inf,group=Period)
              ,size=1,stat='identity',fill="#ef8a62",colour="#ef8a62",alpha=0.5)+
  geom_linerange(aes(x=h,ymax=Q+2*uQ,ymin=Q-2*uQ),data=data.gaug[inter.Gaug.per1,],colour=palette.col[nper1],size=1)+
  geom_point(aes(x=h,y=Q),data=data.gaug[inter.Gaug.per1,],colour=palette.col[nper1],na.rm=na.rm.meas,shape=20,size=2*size.gaugings)+
  xlab(expression(paste("Stage h [m]",sep="")))+
  ylab(expression(paste("Discharge Q [",m^3,".",s^-1,"]",sep="")))+
  coord_cartesian(xlim=xlim.wind,ylim=ylim.wind)+
  scale_y_log10(breaks=breaks.log,labels=labels.log)+
  theme_bw(base_size=40)+
  theme(axis.text=element_text(size=30),axis.title=element_text(size=50,face="bold")
        ,panel.grid.major=element_line(size=1),panel.grid.minor=element_line(size=1.5)
        # ,panel.grid.major=element_blank()
        ,panel.grid.minor.y=element_blank()
        ,legend.text=element_text(size=15),legend.title=element_text(size=30)
        ,legend.key.size=unit(1.5, "cm"),legend.position="none")

pdf(paste(NameStudy,"_per",nper1,".pdf",sep=""),9,8,useDingbats=F)
print(RC_per1)
dev.off()

for(nper1 in 2:5){
  inter.Gaug.per1=which(data.gaug$Period==nper1)
  RC_per1=ggplot(data.RC[which(data.RC$Period==nper1),])+
    geom_vline(xintercept = MaxPost.save[nper1,7],colour="black",size=1)+
    geom_vline(xintercept = MaxPost.save[nper1,12],colour="black",size=1,linetype="longdash")+
    geom_ribbon(aes(x=h,ymax=sup,ymin=inf,group=Period),data=data.RC.prior[which(data.RC.prior$Period==nper1),]
                ,size=1,stat='identity',fill="#67a9cf",colour="#67a9cf",alpha=0.5)+
    geom_ribbon(aes(x=h,ymax=sup,ymin=inf,group=Period)
                ,size=1,stat='identity',fill="#ef8a62",colour="#ef8a62",alpha=0.5)+
    geom_linerange(aes(x=h,ymax=Q+2*uQ,ymin=Q-2*uQ),data=data.gaug[inter.Gaug.per1,],colour="black",size=1)+
    geom_point(aes(x=h,y=Q),data=data.gaug[inter.Gaug.per1,],colour="black",na.rm=na.rm.meas,shape=20,size=2*size.gaugings)+
    xlab(expression(paste("Stage h [m]",sep="")))+
    ylab(expression(paste("Discharge Q [",m^3,".",s^-1,"]",sep="")))+
    coord_cartesian(xlim=xlim.wind,ylim=ylim.wind)+
    scale_y_log10(breaks=breaks.log,labels=labels.log)+
    theme_bw(base_size=40)+
    theme(axis.text=element_text(size=30),axis.title=element_text(size=50,face="bold")
          ,panel.grid.major=element_line(size=1),panel.grid.minor=element_line(size=1.5)
          # ,panel.grid.major=element_blank()
          ,panel.grid.minor.y=element_blank()
          ,legend.text=element_text(size=15),legend.title=element_text(size=30)
          ,legend.key.size=unit(1.5, "cm"),legend.position="none")
  ### Save figure
  pdf(paste(NameStudy,"_per",nper1,".pdf",sep=""),9,8,useDingbats=F)
  print(RC_per1)
  dev.off()
}

#########################################################################################################
############################################## Figure 10 ################################################
#########################################################################################################
##### Loading prior results

##### subplot a

###### layout and save
coef.wind=8;cols=4
layout=NULL
title.num=c("Second", "Third", "Fourth", "Fifth")
###### densities settings
alpha.dens=c(0.5,0.5)
coeff.dens=rep(3,4)
size.gene=2
size.vline=matrix(2,ncol=2,nrow=4)
colour.dens=c("#ef8a62","#67a9cf")
linetype.vline=matrix("longdash",ncol=2,nrow=4)
###### annotate settings
sizeDens=rep(40,4)
coeff.annote=matrix(0.85,ncol=2,nrow=4)
col.annotate=rep("#bfbfbfbf",4)
label=c(as.character(expression(b[1]^(2)))
        ,as.character(expression(b[1]^(3))),as.character(expression(b[1]^(4)))
        ,as.character(expression(b[1]^(5))))
###### theme settings
axis.label=expression(paste("Offset ",b[1]^(2)," [m]",sep=""),paste("Offset ",b[1]^(3)," [m]",sep="")
                      ,paste("Offset ",b[1]^(4)," [m]",sep=""),paste("Offset ",b[1]^(5)," [m]",sep=""))
size.axis.text.x=20;face.axis.text.x="bold";size.axis.title.x=20


data.post.offset=data.MCMC.cooked[,2:5]
data.prior.offset=data.prior[,2:5]

data.mean.prior=as.numeric(prior.mean[2:5])
data.map=as.numeric(data.MCMC.MaxPost[2:5])



###### settings
nsim.post=dim(data.post.offset)[1]
nsim.prior=dim(data.prior.offset)[1]
nvar=dim(data.post.offset)[2]
Quant.MCMC.post=apply(data.post.offset,MARGIN=2,quantile,probs=c(0.025,0.975))
Quant.MCMC.prior=apply(data.prior.offset,MARGIN=2,quantile,probs=c(0.025,0.975))

### layout
numPlots=nvar
# Make the panel
# ncol: Number of columns of plots
# nrow: Number of rows needed, calculated from # of cols
if(is.null(layout)){
  layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                   ncol = cols, nrow = ceiling(numPlots/cols),byrow = TRUE)
}
NameStudy="Meyras_fig8"
##### PDF opening
pdf(paste(NameStudy,"_densities.pdf",sep=""),coef.wind*cols,coef.wind*ceiling(numPlots/cols),useDingbats=F)

# Set up the page
grid.newpage()
pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

for(i in 1:nvar){
  matDist.vect.add=data.frame(x=data.post.offset[,i])
  matDist.vect.nolink=data.frame(x=data.prior.offset[,i])
  min.x=min(density(matDist.vect.add$x)$x,density(matDist.vect.nolink$x)$x)
  max.x=max(density(matDist.vect.add$x)$x,density(matDist.vect.nolink$x)$x)
  min.y=min(density(matDist.vect.add$x)$y,density(matDist.vect.nolink$x)$y)
  max.y=max(density(matDist.vect.add$x)$y,density(matDist.vect.nolink$x)$y)
  
  
  density.post.add=density(matDist.vect.add$x)
  density.post.nolink=density(matDist.vect.nolink$x)
  inter.post.add=which(density.post.add$x>=Quant.MCMC.post[1,i])[1]:(which(density.post.add$x>Quant.MCMC.post[2,i])[1]-1)
  inter.post.nolink=which(density.post.nolink$x>=Quant.MCMC.prior[1,i])[1]:(which(density.post.nolink$x>Quant.MCMC.prior[2,i])[1]-1)
  matPost.dens=data.frame(x.add=density.post.add$x,y.add=density.post.add$y
                          ,x.nolink=density.post.nolink$x,y.nolink=density.post.nolink$y)
  
  m=ggplot(matPost.dens)+geom_hline(yintercept=0,colour="grey90")+
    geom_area(aes(x=x.nolink,y=y.nolink),data=matPost.dens[inter.post.nolink,]
              ,colour=NA,size=size.gene,fill=colour.dens[2],alpha=alpha.dens[2])+
    geom_area(aes(x=x.add,y=y.add),data=matPost.dens[inter.post.add,]
              ,colour=NA,size=size.gene,fill=colour.dens[1],alpha=alpha.dens[1])+
    geom_line(aes(x=x.nolink,y=y.nolink),colour=colour.dens[2],size=size.gene,alpha=1)+
    geom_line(aes(x=x.add,y=y.add),colour=colour.dens[1],size=size.gene,alpha=1)#+
    # geom_vline(xintercept=data.mean.prior[i],color=colour.dens[2],size=size.vline[i,2],linetype=linetype.vline[i,2])+
    # geom_vline(xintercept=data.map[i],color=colour.dens[1],size=size.vline[i,1],linetype=linetype.vline[i,1])
  max.y=1.05*max.y
  min.y=-0.05*max.y
  
  if(i==1){
    min.gen.x=min.x
    min.gen.y=min.y
    max.gen.x=max.x
    max.gen.y=max.y
  }else{
    min.gen.x=min(min.gen.x,min.x)
    min.gen.y=min(min.gen.y,min.y)
    max.gen.x=max(max.gen.x,max.x)
    max.gen.y=max(max.gen.y,max.y)
  }
  # m<-m+coord_cartesian(xlim=c(min.x,max.x),ylim=c(min.y,max.y),expand=FALSE)+
  m<-m+
    annotate("text",x=0.7,y=1.1,label=label[i],colour=col.annotate[i]
             ,size=sizeDens[i],parse=TRUE)+
    eval(substitute(
      xlab(x),list(x=axis.label[i])))+
    coord_cartesian(xlim=c(-3.1,1.5),ylim=c(-0.05,1.45),expand=FALSE)+
    theme_bw(base_size=20)+
    theme(axis.text.x=element_text(size=30),axis.text.y=element_blank()
          ,panel.grid.major=element_line(size=1),panel.grid.minor=element_line(size=0.1)
          ,axis.ticks.y=element_blank(),legend.position="none",axis.title.y=element_blank()
          ,axis.title.x=element_text(size=50,face=face.axis.text.x))
  matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))      
  print(m, vp = viewport(layout.pos.row = matchidx$row,
                         layout.pos.col = matchidx$col))
}
dev.off()
