source('Prior_funk.R') # contains functions used below

#---------------------------------------
# Basic properties of the case study
#---------------------------------------
# 1. Define number of controls, periods and MonteCarlo samples
ncontrol=2;nperiod=60;nsim=100000
# 2. Define which parameters are varying in parameterization (b1,a1,c1,b2,a2,c2)
isVar=c(T,T,F,F,F,F)
# 3. Define parameter names and prior distributions for b,a,c (of any control)
names.bac=c('b','a','c')
margins.bac=c('Gaussian','LogNormal','Gaussian')

#---------------------------------------
# Define priors on "physical" parameters
#---------------------------------------
# Priors are defined through Monte Carlo samples
# 1. Priors for low-flow channel (control 1)
b1<-rnorm(nsim,mean=2,sd=0.5) # channel bed stage
c1<-rnorm(nsim,mean=5/3,sd=0.025) # exponent
Bc1<-rlnorm(nsim,meanlog=log(50),sdlog=0.1) # channel width
KS1<-rlnorm(nsim,meanlog=log(25),sdlog=0.2) # Strickler coefficient
S01<-rlnorm(nsim,meanlog=log(0.00143),sdlog=0.25) # slope
# 2. Priors for high-flow channel (control 2)
b2<-rnorm(nsim,mean=2.5,sd=0.5) # channel bed stage
c2<-rnorm(nsim,mean=5/3,sd=0.025) # exponent
Bc2<-rlnorm(nsim,meanlog=log(200),sdlog=0.1) # channel width
KS2<-rlnorm(nsim,meanlog=log(25),sdlog=0.2) # Strickler coefficient
S02<-rlnorm(nsim,meanlog=log(0.00143),sdlog=0.25) # slope
# 3. Priors for changes on channel bed (control 1)
d.b1<-matrix(rnorm(nsim*(nperiod-1),mean=0,sd=0.2),nrow=nsim,ncol=nperiod-1)
# 4. Priors for changes on channel width (control 1)
d.Bc1<-matrix(rlnorm(nsim*(nperiod-1),meanlog=log(1),sd=0.25),nrow=nsim,ncol=nperiod-1)
# 5. Starting point for all parameters
start=list(
  b1=2,c1=5/3,Bc1=50,KS1=25,S01=0.00143,  # control 1
  b2=2.5,c2=5/3,Bc2=200,KS2=25,S02=0.00143, # control 2
  d.b1=rep(0,nperiod-1),     # changes on channel bed
  d.Bc1=rep(1,nperiod-1)      # changes on channel width 
)

#---------------------------------------
# Perform Monte-Carlo propagation
#---------------------------------------
MC=propagate_Wairau(b1,c1,Bc1,KS1,S01, # control 1: low-flow channel
                    b2,c2,Bc2,KS2,S02, # control 2: high-flow channel
                    d.b1,d.Bc1,        # incremental changes on low-flow channel bed and width
                    start
)

#---------------------------------------
# Fit prior distribution on MC samples
#---------------------------------------
# define marginal prior distributions and parameter names
margins=c();names=c();k=0
for(i in 1:ncontrol){
  for(j in 1:3){
    k=k+1
    if(isVar[k]){
      margins=c(margins,rep(margins.bac[j],nperiod))
      names=c(names,paste(names.bac[j],i,'_',1:nperiod,sep=''))      
    }else{
      margins=c(margins,margins.bac[j])
      names=c(names,paste(names.bac[j],i,sep=''))
    }  
  }
}
# fit multivariate prior distribution
prior=fit(MC$sim,margins,names)
# write config files
colperiod=4 # columns where period will be stored in BaM data file
writeConfigFiles(prior,MC$start,ncontrol,nperiod,colperiod,isVar)
